// Copyright (c) 2005, Rodrigo Braz Monteiro
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of the Aegisub Group nor the names of its contributors
//     may be used to endorse or promote products derived from this software
//     without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
//
// AEGISUB
//
// Website: http://aegisub.cellosoft.com
// Contact: mailto:zeratul@cellosoft.com
//


#pragma once


// Headers
#include "Athenasub.h"
#include "fastbuffer.h"
#include <wx/stream.h>


namespace Athenasub {

	// Text file reader
	class TextFileReader {
		friend class PrefetchThread;
	private:
		wxCriticalSection mutex;

		std::list<String> cache;
		FastBuffer<char> buffer1;
		FastBuffer<wchar_t> buffer2;

		wxString encoding;
		wxInputStream &file;
		shared_ptr<wxMBConv> conv;
		bool Is16;
		bool swap;
		bool trim;
		bool threaded;
		wxThread *thread;

		void SetEncodingConfiguration();
		String ActuallyReadLine();

	public:
		TextFileReader(wxInputStream &stream,String encoding=_T(""),bool trim=true,bool prefetch=true);
		~TextFileReader();

		String ReadLineFromFile();
		bool HasMoreLines();

		static void EnsureValid(const String encoding);
		String GetCurrentEncoding();
		static String GetEncoding(const String filename);
	};

	// Prefetch thread
	class PrefetchThread : public wxThread {
	private:
		TextFileReader *parent;

	public:
		wxThread::ExitCode Entry();
		PrefetchThread(TextFileReader *_parent) : parent(_parent) {}
	};

}

